#import "/global.typ": *

#pagebreak()

= How to Godot
#let wrapleft = [
#link("https://godotengine.org/features/")[Godot] is an open-source game engine under the MIT license.
It uses an object-oriented programming model and has its own scripting language #link("https://docs.godotengine.org/en/stable/tutorials/scripting/gdscript/index.html")[GDScript] with optional typing and python-like syntax.
It also supports #link("https://docs.godotengine.org/en/stable/tutorials/scripting/c_sharp/index.html")[C\#/.NET]
and has native bindings called GDExtensions for C and C++.
There are also several #link("https://github.com/Godot-Languages-Support/godot-lang-support")[community-maintained language bindings] like #link("https://github.com/utopia-rise/godot-kotlin-jvm")[Kotlin]
or #link("https://github.com/Geequlim/ECMAScript")[JavaScript/TypeScript] with a modified base engine or #link("https://docs.godotengine.org/en/stable/tutorials/scripting/gdextension/what_is_gdextension.html#supported-languages")[Rust, Go, and Swift] for GDExtensions.
However, I advise using GDScript or C\# as they have the best support, and have a programming model fitting the engine.
]
#wrap-content(image("/images/godot/icon.svg", width: 4em), wrapleft)

Some general hints for Godot:
- All fields accepting numbers also accept math expressions like `sqrt(2) - 1`.
- You navigate in the 3D panel, by pressing the right mouse button and move around with WASD.
  Press `Shift` to fly faster.

== UI
UI components all inherit from the `Control` class.
As simple but powerful layout containers, I recommend using `BoxContainer`s or `FlowContainer`s,
which work conceptually similar to #link("https://css-tricks.com/snippets/css/a-guide-to-flexbox/")[flexbox] in the HTML document model.
`HBoxContainer` and `VBoxContainer` stretch elements along the cross axis and cannot wrap in main direction.
`HFlowContainer` and `VFlowContainer` don't stretch elements along the cross axis and do wrap in main direction.
Elements whose position and size is not determined by the parent container, can use either absolute positions or anchors.

#figure(image("/images/godot/Anchors.png"), caption: [
  The `VBoxContainer` named `ButtonContainer` in this example is anchored to be in a rectangle with $0.45 dot mono("width")$ as its left border.
  The anchor point rectangle is shown by the green pins in the main view.
  The rectangle after adding the absolute anchor offsets to each border is shown as orange box in the main view.
])

Text colors of `Label`s, space between of container elements, and similar properties are defined by the theme and can be overwritten by individual elements.
To theme your UI, set the theme in the topmost UI node.
#figure(image("/images/godot/Theme.png"), caption: [
  In the right box, you set the theme for a node and all it's children.
  On the bottom panel, you have a preview on the left and a panel to edit the properties (f.e. the text color) of UI types on the right.
])

== Input processing
Godot can handle keyboard, mouse, touchscreen, controllers, joystick, and gamepad inputs.
Controllers, joysticks or gamepads are modelled the same and named _Joypads_ in #link("https://docs.godotengine.org/en/stable/tutorials/inputs/inputevent.html")[the Godot docs].
Touch devices are processed as mouse events with the `device` property set to $-1$.

There are two ways to process inputs.
One method is, to handle inputs by setting actions in #path(("Projects", "Project Settings...", "Input Map (Tab)")),
and then question in `_process` whether `is_action_pressed` or `is_action_just_pressed` on the #link("https://docs.godotengine.org/en/stable/classes/class_input.html")[`Input`] singleton is true.
If you have multiple input devices the action map does not distinguish between them,
so instead you have to check for keys with #link("https://docs.godotengine.org/en/stable/classes/class_input.html#class-input-method-is-key-label-pressed")[`is_key_label_pressed`] or #link("https://docs.godotengine.org/en/stable/classes/class_input.html#class-input-method-is-physical-key-pressed")[`is_physical_key_pressed`], or joypad inputs with #link("https://docs.godotengine.org/en/stable/classes/class_input.html#class-input-method-get-axis")[`get_joy_axis`] or #link("https://docs.godotengine.org/en/stable/classes/class_input.html#class-input-method-is-joy-button-pressed")[`is_joy_button_pressed`].

To know what gamepads are connected you can either query for them with #link("hhttps://docs.godotengine.org/en/stable/classes/class_input.html#class-input-method-get-connected-joypads")[`get_connected_joypads`] or observe the #link("https://docs.godotengine.org/en/stable/classes/class_input.html#signals")[`joy_connection_changed` signal].

As a second method, instead of querying input state, you can also process input events as shown in #link("https://docs.godotengine.org/en/stable/tutorials/inputs/inputevent.html")[the Godot docs].

To change a rigid body's velocity directly (not via impulse or force) based on user input, override `state.linear_velocity` in
#link("https://docs.godotengine.org/en/stable/classes/class_rigidbody3d.html#class-rigidbody3d-private-method-integrate-forces")[`_integrate_forces`], don't change physics state in `_process` or `_physics_process`.

== Physics
You can generate a collider from a `MeshInstance3D` by selecting it, then open the `Mesh` options as shown in @godot-create-mesh.

#figure(image("/images/godot/create_mesh.png"), caption: [
  "`Create Trimesh Static Body`" creates a static body (position and rotation are locked) with a complex collider.
  The other options create dynamic rigid bodies with colliders of varying complexity for this mesh.
]) <godot-create-mesh>

#let simple_mesh_figure = figure(image("/images/godot/simple_collider.png"), caption: [
  Inspector view, with a simple shape (here a sphere) selected
])
<godot-simple-collider>

#let body = [
  Instead of generating colliders, you can also add a child of type `CollisionShape3D` and select a simple shape in its shape attribute.
  This is preferable performance-wise.
]
#wrap-content(simple_mesh_figure, body, align: top + left)

== Networking
See #link("https://docs.godotengine.org/en/stable/tutorials/networking/high_level_multiplayer.html")[the docs] for an introduction to Godot's high-level networking and sample code to build a client-server based lobby.
The `multiplayer` variable is global, and assigning a client or server peer object to it enables multiplayer via the high-level API.

If you want spawned scenes to replicate on spawn across the network, you will need #link("https://docs.godotengine.org/en/stable/classes/class_multiplayerspawner.html")[`MultiplayerSpawner`].
To sync state continuously, you will need a #link("https://docs.godotengine.org/en/stable/classes/class_multiplayersynchronizer.html#class-multiplayersynchronizer")[`MultiplayerSynchronizer`] child for the synchronized node, which has a script, that sets the properties, that should be synced, in the #link("https://docs.godotengine.org/en/stable/classes/class_multiplayersynchronizer.html#class-multiplayersynchronizer-property-replication-config")[`replication_config`].

If you sync the positions of rigid bodies, you should #link("https://docs.godotengine.org/en/latest/classes/class_rigidbody2d.html#class-rigidbody2d-property-freeze")[`freeze`] them in #link("https://docs.godotengine.org/en/latest/classes/class_rigidbody2d.html#enum-rigidbody2d-freezemode")[`FREEZE_MODE_KINEMATIC`] on peers that don't control the body.

== Global state / Keep state across scenes
https://docs.godotengine.org/en/stable/tutorials/scripting/singletons_autoload.html

== How to cross-compile?
To export Godot games to different platforms,
you can read #link("https://docs.godotengine.org/en/stable/tutorials/export/index.html")[their manual],
or follow @godot-export.


#let godotExportGrid = grid(
  columns: 3,
  align: horizon,
  image("/images/godot/export/1 Open Export.png", fit: "contain"),
  image("/images/godot/export/2 Add.png", fit: "contain"),
  image("/images/godot/export/3 Select platform.png", fit: "contain"),
  image("/images/godot/export/4 Add template.png", fit: "contain"),
  image("/images/godot/export/5 Download.png", fit: "contain"),
  image("/images/godot/export/6 Export.png", fit: "contain"),
)
#figure(godotExportGrid, caption: [
  Open #path(("Project", "Export")) for the export menu.
  There you can add targets with `Add`.
  After adding the first target, Godot will complain that you have no export templates.
  By click on `Manage Export Templates` you can open the dialog to download them.
  After downloading the templates, you can add all targets and set the export path for each target.
  You can either export a single target with `Export Project` or all with `Export All...` after configuring the export path for every target.
]) <godot-export>

To use headless mode, simply add the `--headless` option when executing the game.

To run on macOS, #link("https://developer.apple.com/programs/enroll/")[enroll the apple developer program] or
#link("https://docs.godotengine.org/en/stable/tutorials/export/running_on_macos.html")[disable gatekeeper].